#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# conkyRhythmbox-GetCoverart.py is a simple python script to update a known
# cover art path to be refreshed on track change.
#
#  Author: Kaivalagi
# Created: 06/08/2010
import traceback
import shutil
import gobject
import os
import sys
import eyeD3

try:
    import dbus
    DBUS_AVAIL = True
except ImportError:
    # Dummy D-Bus library
    class _Connection:
        get_object = lambda *a: object()
    class _Interface:
        __init__ = lambda *a: None
        ListNames = lambda *a: []
    class Dummy: pass
    dbus = Dummy()
    dbus.Interface = _Interface
    dbus.service = Dummy()
    dbus.service.method = lambda *a: lambda f: f
    dbus.service.Object = object
    dbus.SessionBus = _Connection
    DBUS_AVAIL = False

class Rhythmbox_GetCoverart:

    COVERART_DESTINATION = "/tmp/rhythmbox-coverart.jpg"
    current = ""
    
    
    def __init__(self):
        
        try:
            print "conkyRhythmbox-GetCoverart - Cover art will be copied to '%s'"%self.COVERART_DESTINATION
            print "* Initialising: Creating a D-Bus session..."
            self.bus = dbus.SessionBus()
            self.remote_object_shell = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Shell')
            self.iface_shell = dbus.Interface(self.remote_object_shell, 'org.gnome.Rhythmbox.Shell')
            self.remote_object_player = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Player')
            self.iface_player = dbus.Interface(self.remote_object_player, 'org.gnome.Rhythmbox.Player')
            self.iface_player.connect_to_signal("playingUriChange", self.OnTrackChange)
            print "* Listening: waiting for a track change..."
        except:
            print "** Error: Is Rhythmbox running? The D-Bus service is not working."
            traceback.print_exc()
    
    def OnTrackChange(self,sender=None):
        try:
            
            self.props = iface_shell.getSongProperties(iface_player.getPlayingUri())
            
            src = self.getCoverartPath()
            
            if src == None:
                
                print "** Track Changed: No cover art images known, Checking for embedded images in audio file..."
                
                if len(self.getCoverartImage(self.COVERART_DESTINATION)) > 0:
                    print "**                Image found and extracted to %s."%self.COVERART_DESTINATION
                else:
                    print "**                No image found."
                    
            elif src == self.current:
                
                print "** Track changed: Cover art unchanged, no copy necessary"

            elif os.path.exists(src) == True:
                
                print "** Track changed: Copying '%s' to '%s'"%(src,self.COVERART_DESTINATION)
                shutil.copy(src, self.COVERART_DESTINATION)
                self.current = src
                                    
            else:
                print "** Track changed: No sure what to do: src = '%s', dst='%s'"%(src,self.COVERART_DESTINATION) 
        except:
            print "*** Error: Begin"
            traceback.print_exc()
            print "*** Error: End"
        finally:
            print "* Listening: waiting for a track change..."
                        
    def getCoverartPath(self):
        
        try:
            
            # get coverart url or file link
            coverart = None
            if "rb:coverArt-uri" in props:
                coverart = props["rb:coverArt-uri"]
                if coverart.find("http://") != -1:
                    coverart = None
                else:
                    coverart = coverart.replace("file://","")
                
            return coverart
        except:
            traceback.print_exc()
            
    def getCoverartImage(self,savepath):

        # get file apth
        if "location" in self.props:
            location = self.props["location"]
        
        srcfilepath = location.replace("file://","").replace("%20"," ")
        dstfolder = os.path.dirname(savepath) 
        dstfile = savepath.replace(os.path.dirname(savepath)+"/","")
        
        tag = eyeD3.Tag()
        
        if os.path.isfile(srcfilepath):
            if os.path.splitext(srcfilepath)[1] == ".mp3":
                # Extract picture
                try:
                    tag.link(srcfilepath)
                    for image in tag.getImages():
                        image.writeFile(dstfolder, dstfile)
                        return savepath
                except:
                    print "Unable to extract image for: " + srcfilepath
                    traceback.print_exc()
        return ""


if __name__ == '__main__':
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)
    loop = gobject.MainLoop()
    Rhythmbox_GetCoverart()
    loop.run()
    
